{% extends "layout.html" %}

{% block url %}
    {% if url == None %}
        ..
    {% else %}
        {{ url }}
    {% endif %}
{% endblock %}

{% block body %}
    
<div class="container theme-showcase" role="main">
	{% with messages = get_flashed_messages(with_categories=true) %}
		{% if messages %}
			<dl class=flashes>
				{% for category, message in messages %}
                    {% if category == 'message' %}
                    	{% set category = 'warning' %}
                    {% endif %}
                    <li class="label label-{{ category }}">{{ message }}</li>
                {% endfor %}
			</dl>
		{% endif %}
	{% endwith %}
    <div class="row"><br><br><br></div>
	<div class="row">
	    <div class="col-md-8">
		    <h1>About</h1>
		    <p>
		    This website is built with poets, the Python Open Earth Observation Tools. 
		    <br>
		    poets is a package with aims to provide a standard library that can be used 
		    for collecting and resampling geospatial image datasets.
		    </p>
		    <p>It is licensed under
		    <a href="https://github.com/TUW-GEO/poets/blob/master/LICENSE.txt" target="_blank">BSD</a>.
		    Feel free to fork the code on <a href="https://github.com/TUW-GEO/poets" target="_blank">GitHub</a> and contribute to the project.
		    Check <a href="http://rs.geo.tuwien.ac.at/poets" target="_blank">the documentation</a> for more information and installation instructions.
		    The package is available at the <a href="https://pypi.python.org/pypi/poets" target="_blank">Python Package Index</a>.
		    </p>
		    <h3>built with...</h3>
	        <ul>
	            <li><a href="http://flask.pocoo.org/" target="_blank" class="hi">Flask</a> <em>a microframework for Python based on Werkzeug, Jinja 2 and good intentions.</em></li>
	            <li><a href="http://getbootstrap.com/" target="_blank" class="hi">Bootstrap</a> <em>the most popular HTML, CSS, and JS framework for developing responsive, mobile first projects on the web.</em></li>
	            <li><a href="http://openlayers.org/" target="_blank" class="hi">OpenLayers</a> <em>A high-performance, feature-packed library for all your mapping needs.</em></li>
	            <li><a href="http://dygraphs.com/" target="_blank" class="hi">dygraphs</a> <em>a fast, flexible open source JavaScript charting library.</em></li>
	        </ul>
	    </div>
	    <div class="col-md-4">
	        <h1>poets</h1>
	        <div class="bs-callout bs-callout-danger">
	            <h4>Documentation</h4>
	            <a href="http://rs.geo.tuwien.ac.at/poets" target="_blank">http://rs.geo.tuwien.ac.at/poets</a>
	        </div>
	        <div class="bs-callout bs-callout-info">
	            <h4>Source code</h4>
	            <a href="https://github.com/TUW-GEO/poets" target="_blank">https://github.com/TUW-GEO/poets</a>
	        </div>
	        <div class="bs-callout bs-callout-warning">
	            <h4>Python Package</h4>
	            <a href="https://pypi.python.org/pypi/poets" target="_blank">https://pypi.python.org/pypi/poets</a>
	        </div>
	    </div>
	</div>
</div>


{% endblock %}