<!DOCTYPE html>
<html lang="en" style="height:100%;">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>Web Interface {% block title %}{% endblock %}</title>
    
    <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">

    <!-- Bootstrap -->
    <link rel="stylesheet" href="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/css/bootstrap.min.css">
    <!-- Bootstrap Theme -->
    <link rel="stylesheet" href="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/css/bootstrap-theme.min.css">
    <!-- OpenLayers Theme -->
    <link rel="stylesheet" href="static/ol2.13.1/theme/default/style.css" type="text/css">
    <!-- Bootstrap Slider Theme -->
    <link rel="stylesheet" href="static/slider/css/slider.css">
	<!--[if lt IE 7]><link rel="stylesheet" href="css/font-awesome-ie7.min.css"><![endif]-->

    <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js"></script>
      <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
    <![endif]-->
    
    <!--  poets -->
    <script src="static/js/poets.js"></script>
    
    <!-- Dygraphs -->
    <script src="static/js/dygraph-combined.js"></script>
    
    <!-- jQuery -->
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
  
  </head> 
  
  <body role="document" style="height:100%;">
  
	<div class="navbar navbar-inverse navbar-fixed-top" role="navigation">
	    <div class="container">
	      <div class="navbar-header">
	        <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
	          <span class="sr-only">Toggle navigation</span>
	          <span class="icon-bar"></span>
	          <span class="icon-bar"></span>
	          <span class="icon-bar"></span>
	        </button>
	        <a class="navbar-brand" rel="home" href="" title="poets°">
<!-- 		        <img style="max-width:100px; margin-top: -7px;" -->
<!-- 		             src="{{ url_for('static', filename='poets_logo_white_32.png') }}"> -->
					<img style="max-width:100px; margin-top: -7px;"
                     src="static/poets_logo_white_32.png">
		    </a>
	      </div>
	      <div class="collapse navbar-collapse">
	        <ul class="nav navbar-nav">
              <li class="active"><a href="{% block url %}{% endblock %}">Home</a></li>
              <li><a href="about">About</a></li>
            </ul>
	      </div><!--/.nav-collapse -->
	  </div>
	</div>

    <div class="container" style="height:100%;">
        {% with messages = get_flashed_messages(with_categories=true) %}
	        {% if messages %}
	            <dl class=flashes>
	                {% for category, message in messages %}
	                    {% if category == 'message' %}
	                       {% set category = 'warning' %}
	                    {% endif %}
	                    <li class="label label-{{ category }}">{{ message }}</li>
	                {% endfor %}
	            </dl>
	        {% endif %}
	    {% endwith %}

		<div class="row"><br><br><br></div>
        {% block body %}{% endblock %}
    </div>
    
    <!-- Google Maps API -->
    <script src='http://maps.google.com/maps/api/js?v=3.2&amp;sensor=false' type='text/javascript'></script>

    <!-- jQuery (necessary for Bootstrap's JavaScript plugins) -->
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
    
    <!-- Include all compiled plugins (below), or include individual files as needed -->
    <script src="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/js/bootstrap.min.js"></script>
    
    <!-- OpenLayers -->
    <script src='static/ol2.13.1/OpenLayers.js' type='text/javascript'></script>
    
    <!-- Bootstrap slider -->
    <script src="static/slider/js/bootstrap-slider.js"></script>
  </body>
</html>